import Table from "cli-table";
import StoragePlace from "./enum";
import Product from "./Products/Product";
import Corn from "./Products/Corn";
import Fish from "./Products/Fish";
import Milk from "./Products/Milk";
import Salt from "./Products/Salt";
import Stew from "./Products/Stew";
import { getRandomNumber } from "../helpers";

class Store {
    public products: Product[] = [];

    constructor() {
        for(let i = 0; i < 100; i++) {
            const products = [
                new Corn({deliveryTimestamp: this.getRandomDeliveryTime(), storagePlace: this.getRandomStoragePlace()}), 
                new Fish({deliveryTimestamp: this.getRandomDeliveryTime(), storagePlace: this.getRandomStoragePlace()}), 
                new Milk({deliveryTimestamp: this.getRandomDeliveryTime(), storagePlace: this.getRandomStoragePlace()}), 
                new Salt({deliveryTimestamp: this.getRandomDeliveryTime(), storagePlace: this.getRandomStoragePlace()}), 
                new Stew({deliveryTimestamp: this.getRandomDeliveryTime(), storagePlace: this.getRandomStoragePlace()})
            ];
            const randomIndex = getRandomNumber({min: 0, max: products.length - 1});
            this.products.push(products[randomIndex]);
        };
    };

    public doInspection(): void {
        const table = new Table({
            head: ['Product', 'Delivered at', 'Storage place', 'S. life days', 'Fresh'], 
            colWidths: [20, 20, 15, 15, 10]
        });

        this.products?.forEach(product => {
            table.push([
                product.name,
                product.deliveryTimestamp?.toISOString(),
                product.storagePlace,
                product.storageLifeDays?.toString(),
                product.isFresh() ? "Fresh" : "Not Fresh"
            ]);
        });

        console.log(table.toString());
    };

    public getRandomDeliveryTime(): Date {
        const randomNum: number = getRandomNumber({min: 1, max: 200});
        return new Date(Date.now() - 86400000 * randomNum);
    };

    public getRandomStoragePlace(): StoragePlace {
        const enums: (string | StoragePlace)[] = Object.values(StoragePlace);
        const randomIndex = Math.floor(Math.random() * enums.length);
        const randomEnumKey = enums[randomIndex];
        return randomEnumKey as StoragePlace;
    };
};

export default Store;